/*	DOS Stamp Power Save demo program
	Copyright 1999 Bagotronix Inc.

	This program shows how to conserve power with the AM188ES CPU on the
	DOS Stamp.

	v0.99	07/31/99 Ivan Baggett

	DOS Stamp current draw (approximate) at 5 V for various power save modes:
	full speed:		200mA
	1/2 speed:		120mA
	1/4 speed:		 80mA
	1/8 speed:		 60mA
	1/16 speed:		 50mA
	1/32 speed:		 40mA
	1/64 speed:		 40mA
	1/128 speed:	 40mA

	As you can see, going below 1/16 speed doesn't get you much.
*/

#include <stdio.h>
#include <dos.h>
#include "stdinc.h"
#include "dosstamp.h"

#define	DSCLK		0	// full speed
#define DSCLK_2		1	// 1/2 speed
#define DSCLK_4		2	// 1/4 speed
#define DSCLK_8		3	// 1/8 speed
#define DSCLK_16	4	// 1/16 speed
#define DSCLK_32	5	// 1/32 speed
#define DSCLK_64	6	// 1/64 speed
#define DSCLK_128	7	// 1/128 speed

#define TICK_VEC	0x08	// BIOS Tick vector (PC BIOS standard)


// global variables
ULONG ticks;
UWORD speed;
void interrupt (*oldvect)();
UWORD tmr0cmpa;
UWORD ser0bdiv;


/*	Sets CPU speed, which also affects UARTs and Timers
	Arguments:
		speedmode = 0-7
*/
void setspeed (UWORD speedmode) {
	UWORD spdmode;

	spdmode = speedmode & 0x0007;

	disable();
	// set clock divisor and enable power save
	outport (SYSCON, (inport (SYSCON) & ~0x0007) | spdmode | 0x8000);

	enable();
}


/*	Arguments:
		bauddiv = the baud divisor to use for full speed mode
		speedmode = 0-7
*/
void setUARTspeed (UWORD bauddiv, UWORD speedmode) {
	// set UART baud divisors
	outport (SER0_BDIV, bauddiv / (1 << (speedmode & 0x0007)));
}


/*	Arguments:
		normcount = the compare value to use for full speed mode
		speedmode = 0-7
*/
void settimerspeed (UWORD normcount, UWORD speedmode) {
	// set Timer 0 max count A register
	outport (TMR0_CMPA, normcount / (1 << (speedmode & 0x0007)));
}


/*	Chain into the BIOS Timer Tick interrupt (INT 08h), which is called when
	Timer 0 reaches max count.  This is standard PC BIOS practice.

	When programmed for low speeds, any hardware interrupt events cause the
	CPU to switch to high speed.  Once the ISR is complete, the CPU stays in
	high speed mode unless a low speed mode was programmed inside the ISR.
	Therefore, to return to low speed mode, you need each hardware ISR to
	reprogram the low speed mode.

	Programming for low speed mode affects peripherals that use a derivative
	of the CPU clock.  Therefore, UART baud rate divisors will need to be
	changed, as well as any timer maximum count registers.  Programming the
	UART divisors while there is UART activity will probably result in a few
	garbled characters, so keep this in mind when designing your protocols.
	If you need to conserve power while waiting for receipt of a serial
	transmission, you should use low speed mode until the first character
	comes in, which will cause an interrupt if the UART interrupt is enabled.
	Then stay in full speed mode until your transmission is
	complete.

	If your application cannot tolerate some garbled characters when the CPU
	core is switching speeds, you will have to operate at full speed always.
*/
void interrupt powersave (void) {
	setspeed (speed);
	setUARTspeed (ser0bdiv, speed);
	settimerspeed (tmr0cmpa, speed);
	++ticks;
	oldvect();	// call old BIOS Timer Tick ISR
}


int main (void) {
	BOOLEAN done = FALSE;
	ULONG lticks;
	UWORD lspeed;
	BYTE s[128];
	WORD c;

	printf ("\nDOS Stamp power save demo program");
	printf ("\nSome characters will be garbled due to switching speeds.");
	printf ("\nPress 'X' to quit, other key to experiment...");

	ticks = 0L;
	speed = 0;		// default to full speed
	ser0bdiv = inport (SER0_BDIV);
	tmr0cmpa = inport (TMR0_CMPA);

	// take over User Tick vector
	disable();
	oldvect = getvect (TICK_VEC);
	setvect (TICK_VEC, powersave);
	enable();

	while (!done) {
		if (kbhit()) {
			c = getch();
			if (!c)
				c = getch();
			if (c == 'x' || c == 'X')
				done = TRUE;
			else {
				disable();
				lticks = ticks;
				enable();
				printf ("\nTimer 0 ticks = %lu", lticks);
				printf ("\nEnter new power save mode (0-7): ");
				gets (s);
				sscanf (s, "%u", &lspeed);
				if (lspeed <= 7) {
					disable();
					speed = lspeed;
					enable();
				}
			}
		}
	}

	// restore full speed
	disable();
	speed = DSCLK;
	enable();

	// wait for it to take effect
	// compiler's delay() function does not work on DOS Stamp
	for (c = 0; c < 32000; c++);
	for (c = 0; c < 32000; c++);
	for (c = 0; c < 32000; c++);
	for (c = 0; c < 32000; c++);

	// restore old User Tick vector
	disable();
	setvect (TICK_VEC, oldvect);
	enable();

	return 0;
}
